#========================================================================================
# package      DE_EPAGES::DHLIntraship::t::WebServiceClient_Internal
# state        public
#----------------------------------------------------------------------------------------
# description  Test of WebService Client
#========================================================================================
package DE_EPAGES::DHLIntraship::t::WebServiceClient_Internal;
use strict;

use LWP::UserAgent;
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer;
use DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Test::Mock::EpagesObject;
use HTTP::Response;
use HTTP::Headers;
use Test::MockObject;
use Test::MockObject::Extends;
use Test::More;

#----------------------------------------------------------------------------------------
sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {
            testRequest();
            testResponseOk();
            testResponseNotOk();
            testHttpError();
            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub testRequest {
    my $subTestName = (caller(0))[3];

    my $TestResponse = _ImportTestXmlFile('testResponseOk.xml');
    my $HttpRespone = _CreatePositiveXMLHttpResponse($TestResponse);
    my $UserAgent = _createLWPUserAgent($HttpRespone);
    my $MockedWebServiceClient = _CreateMockedWebServiceClient($UserAgent);
    my $MockedShippingLabel = _CreateMockedShippingLabel();

    $MockedWebServiceClient->createShipmentDD($MockedShippingLabel);

    my $RequestContent = ${$UserAgent->{'HTTPRequest'}->{'_content'}};
    $RequestContent = _NormalizeXmlString($RequestContent);
    my $ExpectedValue = _ImportTestXmlFile('testRequest.xml');
    is($RequestContent, $ExpectedValue, "$subTestName - Test request");
}

#----------------------------------------------------------------------------------------
sub testResponseOk {
    my $subTestName = (caller(0))[3];

    my $TestResponse = _ImportTestXmlFile('testResponseOk.xml');
    my $HttpRespone = _CreatePositiveXMLHttpResponse($TestResponse);
    my $UserAgent = _createLWPUserAgent($HttpRespone);
    my $MockedWebServiceClient = _CreateMockedWebServiceClient($UserAgent);
    my $MockedShippingLabel = _CreateMockedShippingLabel();

    my $aResult = $MockedWebServiceClient->createShipmentDD($MockedShippingLabel);

    is_deeply($aResult, [
          {
            'StatusCode' => '0',
            'ShipmentNumber' => '00340433863111302330',
            'SequenceNumber' => '18666',
            'InvalidAddress' => '1',
            'Labelurl' => 'http://test-intraship.dhl.com:80/cartridge.61/WSPrint?code=65B2127A3E0D61E6C5B345840824150FC201A26B46207FEA',
            'StatusMessage' => 'ok'
          }
        ], "$subTestName - Test Response OK of mocked User Agent");
}

#----------------------------------------------------------------------------------------
sub testResponseNotOk {
    my $subTestName = (caller(0))[3];

    my $TestResponse = _ImportTestXmlFile('testResponseNotOk.xml');
    my $HttpRespone = _CreatePositiveXMLHttpResponse($TestResponse);
    my $UserAgent = _createLWPUserAgent($HttpRespone);
    my $MockedWebServiceClient = _CreateMockedWebServiceClient($UserAgent);
    my $MockedShippingLabel = _CreateMockedShippingLabel();

    eval {
        $MockedWebServiceClient->createShipmentDD($MockedShippingLabel);
    };
    if (ExistsError()) {
        my $Error = GetError();
        like($Error->code(), qr/DHLRespondedWithError/, "$subTestName - Test Response Not OK of mocked User Agent");
    }
}

#----------------------------------------------------------------------------------------
sub testHttpError {
    my $subTestName = (caller(0))[3];

    my $HttpRespone = _CreateNegativeHttpResponse();
    my $UserAgent = _createLWPUserAgent($HttpRespone);
    my $MockedWebServiceClient = _CreateMockedWebServiceClient($UserAgent);
    my $MockedShippingLabel = _CreateMockedShippingLabel();

    my $ErrorCode = '';
    eval {
        $MockedWebServiceClient->createShipmentDD($MockedShippingLabel);
    };
    if (ExistsError()) {
        my $Error = GetError();
        $ErrorCode =$Error->code();
    }
    like($ErrorCode, qr/HttpError/, "$subTestName - Test Http Error of mocked User Agent");
}

#----------------------------------------------------------------------------------------
sub _CreateMockedWebServiceClient {
    my ($UserAgent) = @_;

    my $BuildTransportLayer = DE_EPAGES::DHLIntraship::API::SOAP::BuildTransportLayer->new();
    my $MockedBuildTransportLayer = Test::MockObject::Extends->new( $BuildTransportLayer );
    $MockedBuildTransportLayer->mock('_getLWPUserAgent', sub {
        return $UserAgent;
    });

    my $WebServiceClient = DE_EPAGES::DHLIntraship::API::SOAP::WebServiceClient->new();
    my $MockedWebServiceClient = Test::MockObject::Extends->new( $WebServiceClient );
    $MockedWebServiceClient->mock('createTransportLayer', sub {
        return $MockedBuildTransportLayer->getTransportLayer();
    });
    return $MockedWebServiceClient;
}

#----------------------------------------------------------------------------------------
sub _createLWPUserAgent {
    my ($HttpResponse) = @_;

    my $LWPUserAgent = LWP::UserAgent->new();
    my $MockedUserAgent = Test::MockObject::Extends->new( $LWPUserAgent );
    $MockedUserAgent->mock('request',sub{
                my $self = shift;
                my ($HTTPRequest) = @_;
                $self->{'HTTPRequest'} = $HTTPRequest; #store as field for unit test
                return $HttpResponse;
        });

    return $MockedUserAgent;
}

#----------------------------------------------------------------------------------------
sub _CreateNegativeHttpResponse {
    my $HtmlNegativStatusCode = 500; ## no critic (ProhibitMagicNumbers)
    return HTTP::Response->new( $HtmlNegativStatusCode, 'not OK');
}

#----------------------------------------------------------------------------------------
sub _CreatePositiveXMLHttpResponse {
    my ($Content) = @_;
    my $Headers = HTTP::Headers->new();
    $Headers->header('Content-Type' => 'text/xml');
    my $HtmlPositiveStatusCode = 200; ## no critic (ProhibitMagicNumbers)
    return HTTP::Response->new( $HtmlPositiveStatusCode, 'OK', $Headers, $Content );
}

#----------------------------------------------------------------------------------------
sub _ImportTestXmlFile {
    my ($fileName) = @_;

    my $TestResponsePath = "$ENV{EPAGES_CARTRIDGES}/DE_EPAGES/DHLIntraship/t/TestResponses/";
    my $xmlFile = ${GetFileContent($TestResponsePath.$fileName)};
    $xmlFile = _NormalizeXmlString($xmlFile);

    return $xmlFile;
}

#----------------------------------------------------------------------------------------
sub _NormalizeXmlString {
    my ($XmlString) = @_;
    $XmlString =~ s/\n//g;   # removes new line sign
    $XmlString =~ s/\r//g;   # removes return sign
    $XmlString =~ s/\s+/ /g; # replaces multiple whitespaces with a single whitespace
    $XmlString =~ s/\s</</g; # removes withspaces before '<'-tag
    return $XmlString;
}

#----------------------------------------------------------------------------------------
sub _CreateMockedShippingLabel {

    my $EstimatedDeliveryDate = DateTime->new(
        year       => 2222,
        month      => 2,
        day        => 22,
        hour       => 22,
        minute     => 22,
        );

    my $TestAddress = _GetTestAddress();
    my $FulfillmentMethod = _GetShippingFulfillmentMethodDHLIntraship();
    my $LineItemContainer = _GetLineItemContainer();

    my $hAttributes = {
        'Alias' => '1000-1',
        'ID' => '197',
        'ParcelDepth' => '1',
        'ParcelLength' => '1',
        'ParcelWidth' => '1',
        'TotalWeight' => '1',
        'EstimatedDeliveryDate' => $EstimatedDeliveryDate,
        'LineItemContainer' => $LineItemContainer,
        'ShopAddress' => $TestAddress,
        'Address' => $TestAddress,
        'ShippingFulfillmentMethod' => $FulfillmentMethod,
    };

    my $MockedShippingLabel = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::DHLIntraship::API::Object::ShippingLabelDHLIntraship', $hAttributes);
    $MockedShippingLabel->mock('folder', sub {return _GetFolder();});

    return $MockedShippingLabel;
}

#----------------------------------------------------------------------------------------
sub _GetShippingFulfillmentMethodDHLIntraship {
    my $hAttributes = {
        'CustomerNumber' => '12345678900176',
        'WebserviceUsername' => 'WebserviceUsername',
        'WebservicePassword' => 'WebservicePassword',
    };
    my $FulfillmentMethod = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::DHLIntraship::API::Object::ShippingFulfillmentMethodDHLIntraship', $hAttributes);
    return $FulfillmentMethod;
}

#----------------------------------------------------------------------------------------
sub _GetLineItemContainer {
    my $hAttributes = {
        'LineItemsSum' => '1',
    };
    my $LineItemContainer = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $hAttributes);
    return $LineItemContainer;
}

#----------------------------------------------------------------------------------------
sub _GetFolder() {
    my $Folder = Test::MockObject->new();
    $Folder->mock('children', sub{return;});
    return $Folder;
}

#----------------------------------------------------------------------------------------
sub _GetTestAddress {
    my $hCountry = {
        'Code2' => 'DE',
    };

    my $hAttributes = {
        'Alias' => 'Address',
        'FirstName' => 'FirstName',
        'MiddleName' => 'MiddleName',
        'LastName' => 'LastName',
        'FullName' => 'FullName',
        'City' => 'City',
        'Fax' => '245346546',
        'StreetOnly' => 'StreetOnly',
        'Salutation' => 'Salutation',
        'JobTitle' => 'JobTitle',
        'Company' => 'Company',
        'Country' => $hCountry,
        'EMail' => 'EMail',
        'PhoneCell' => 1,
        'Phone' => 1,
        'HouseNumber' => 1,
        'Zipcode' => 10537,
        'Code2' => 'DE',
    };
    my $Address = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Address::API::Object::Address', $hAttributes);

    return $Address;
}

RunScript( Sub => \&Main );