#========================================================================================
# package      DE_EPAGES::Klarna::t::Activate_internal
# state        public
#----------------------------------------------------------------------------------------
# description  Internal unit tests for Klarna API Activate
#========================================================================================
package Activate_internal;
use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Klarna::API::KlarnaOperations::Activate;
use DE_EPAGES::Klarna::UI::BasketOrder;
use DE_EPAGES::Test::Mock::EpagesObject;
use LWP::UserAgent;
use Test::MockObject::Extends;
use Test::More;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            testRequest();

            testPositiveResponse();
            testFaultResponse();
            testHTTPErrorResponse();

            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub testRequest {
    my $subTestName = (caller(0))[3];

    my $LineItemPayment = _createLineItemPayment();
    my $PointerToLWPUserAgentObject = _createLWPUserAgent('activate_dummy_response.xml');
    my $Activate = _createActivate($LineItemPayment,$PointerToLWPUserAgentObject);
    $Activate->call();

    my $RequestedContent = $PointerToLWPUserAgentObject->{'HTTPRequest'}->content;
    my $ExpectedContent = _getExpectedContent();
    is($RequestedContent,$ExpectedContent
        ,"$subTestName - compare the request structures"
    );
}

#----------------------------------------------------------------------------------------
sub _createActivate {
    my ($LineItemPayment, $LWPUserAgent) = @_;
    my $ActivateObject = DE_EPAGES::Klarna::API::KlarnaOperations::Activate->new($LineItemPayment, $LWPUserAgent);
     my $ActivateObjectMock = Test::MockObject::Extends->new( $ActivateObject );
     $ActivateObjectMock->mock('_LoadRootObject', sub{return _createSystemObject();}); #overwrite LoadRootObject
    return $ActivateObjectMock;
}

#----------------------------------------------------------------------------------------
sub testPositiveResponse {
    my $subTestName = (caller(0))[3];

    my $PointerToLWPUserAgentObject = _createLWPUserAgent('activate_positive_response.xml');
    my $LineItemPayment = _createLineItemPayment();

    my $Activate = _createActivate($LineItemPayment, $PointerToLWPUserAgentObject);
    my $hResponse = $Activate->call();

    my $ExpectedResponse = _getExpectedPositiveResponse();
    is_deeply($hResponse, $ExpectedResponse, "$subTestName - positive response (activate_positive_response.xml)");
}
#----------------------------------------------------------------------------------------
sub testFaultResponse {
    my $subTestName = (caller(0))[3];

    my $PointerToLWPUserAgentObject = _createLWPUserAgent('fault_rno_response.xml');
    my $LineItemPayment = _createLineItemPayment();

    my $Activate = _createActivate($LineItemPayment, $PointerToLWPUserAgentObject);
    my $hResponse = $Activate->call();

    my $ExpectedResponse = _getExpectedFaultResponse();

    is_deeply($hResponse, $ExpectedResponse, "$subTestName - fault response (fault_rno_response.xml)");
}

#----------------------------------------------------------------------------------------
sub testHTTPErrorResponse {
    my $subTestName = (caller(0))[3];

    my $PointerToLWPUserAgentObject = _createLWPUserAgent();
    my $LineItemPayment = _createLineItemPayment();

    my $Activate = _createActivate($LineItemPayment, $PointerToLWPUserAgentObject);
    eval {
        $Activate->call();
    };
    if (ExistsError()) {
        my $Error = GetError();
        like ($Error->shortMessage(), qr/File does not exist:/, "$subTestName - HTTP error response");
    }
}

#========================================================================================
# DAL Wrapper
#========================================================================================
sub _createSystemObject{
    my $hSystemAttributes = {
    'EpagesVersion' => '6.17.0',
    'ProviderID' => 'epages',
    'StoreName' => 'Store',
    'KlarnaProtocolVersion' => '4.1',
    'KlarnaHostTest' => 'http://payment.testdrive.klarna.com',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site',$hSystemAttributes);
}

#----------------------------------------------------------------------------------------
sub _createShopObject {
        my $Attributes = {
        'IsInternalTestShop' => 1,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Store::API::Object::Store',$Attributes);
}

#----------------------------------------------------------------------------------------
sub _createLineItemPayment {
    my $Attributes = {
        'ReservationNumber' => 2347010070,
        'PaymentMethod' => _createPaymentMethod(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer',$Attributes);
}

#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
        my $Attributes = {
            'IsInTestMode' => 1,
            'Kr_StoreID' => 301,
            'Kr_SecretKey' => 'NoKn3kdeMKhxVmP',
            'PaymentType' => _createPaymentType(),
            'Site' => _createShopObject(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaInvoice',$Attributes);
}

#----------------------------------------------------------------------------------------
sub _createPaymentType {
        my $Attributes = {
        'Alias' => 'KlarnaInvoice'
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Payment::API::Attributes::System',$Attributes);
}

#========================================================================================
# Webservice Wrapper
#========================================================================================

sub _createLWPUserAgent {
    my ($RequestFile) = @_;
    my $GetTestResponsePath = GetKlarnaTestResponseDirectory();
    my $LWPUserAgent = LWP::UserAgent->new();
    my $MockedUserAgent = Test::MockObject::Extends->new( $LWPUserAgent );
    $MockedUserAgent->mock('request',sub{
                my $self = shift;
                my ($HTTPRequest) = @_;
                $MockedUserAgent->{'HTTPRequest'} = $HTTPRequest;
                return $self;
        });
    $MockedUserAgent->mock('content',sub{
            my $self = shift;
            my ($Params) = @_;
            if (isValid($RequestFile)){
                return ${GetFileContent($GetTestResponsePath.$RequestFile)};
            }else{
                return " \n";
            }
        });
    return $MockedUserAgent;
}

#----------------------------------------------------------------------------------------
sub GetKlarnaTestResponseDirectory {
    my $CartridgePackage = 'DE_EPAGES::Klarna';
    $CartridgePackage =~ s/::/\//g;
    return "$ENV{EPAGES_CARTRIDGES}/$CartridgePackage/t/TestResponses/";
}

#----------------------------------------------------------------------------------------
sub _getExpectedContent {
    my $ExpectedContent = <<"REQUEST";
<?xml version="1.0" encoding="iso-8859-1"?>
<methodCall>
  <methodName>activate</methodName>
  <params>
    <param>
      <value>
        <string>4.1</string>
      </value>
    </param>
    <param>
      <value>
        <string>perl:api:2.3.0:epages:6.17.0:epages,Store</string>
      </value>
    </param>
    <param>
      <value>
        <int>301</int>
      </value>
    </param>
    <param>
      <value>
        <string>2Hm3ymkfLG3hHS8eFyNXBg==</string>
      </value>
    </param>
    <param>
      <value>
        <string>2347010070</string>
      </value>
    </param>
    <param>
      <value>
        <array>
          <data></data>
        </array>
      </value>
    </param>
  </params>
</methodCall>
REQUEST
;
return $ExpectedContent;
}

#----------------------------------------------------------------------------------------
sub _getExpectedPositiveResponse {
    my $ExpectedPositiveResponse = {
          'StatusCode' => 'OK',
          'Message' => [
                         {
                           'string' => 'ok'
                         },
                         {
                           'string' => '1000530750434'
                         }
                       ]
    };
return $ExpectedPositiveResponse;
}

#----------------------------------------------------------------------------------------
sub _getExpectedFaultResponse {
    my $ExtectedFaultResponse = {
          'StatusCode' => '9109',
          'Message' => 'A problem has occured in the communication to Klarna that handles our invoicing. ' .
                       'Please contact Klarna (http://www.klarna.com) for more information or choose another payment method.'
        };
return $ExtectedFaultResponse;
}

RunScript( Sub => \&Main );
1;