#========================================================================================
# package      DE_EPAGES::Klarna::t::CancelReservation_External
# state        public
#----------------------------------------------------------------------------------------
# description  External integration tests for Klarna API CancelReservation
#========================================================================================
package DE_EPAGES::Klarna::t::CancelReservation_External;

use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Object::API::Factory qw (
    ExistsObjectByPath
    LoadObjectByPath
    LoadRootObject
);
use DE_EPAGES::Object::API::XML::ObjectImportHandler qw ( ImportObjects );
use DE_EPAGES::Presentation::API::Session qw ( InsertSession );
use DE_EPAGES::Presentation::API::Table::Session qw ( GetInfoSessionBySessionID );
use DE_EPAGES::WebInterface::API::FakeRequest;
use Readonly;
use Test::More;

Readonly my $ROOTPATH => '/Test';
Readonly my $SHOPPATH => $ROOTPATH.'/TestShopKlarna';
Readonly my $ORDERPATH => $SHOPPATH.'/Customers/TestCustomer/Orders/1234-5678';

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            # Test
            eval {
                if (not $ENV{'SKIP_EXTERNAL_TESTS'}){
                    ok( Init(), 'initialization' );
                    Test_ReserveAmount();
                    Test_CancelReservation();
                } else {
                    plan skip_all => 'call Klarna Server (SKIP_EXTERNAL_TESTS == 1)';
                }
            };
            if ( ExistsError() ) {
                my $Error = GetError();
                fail( 'Error: ' . $Error->longMessage );
            }

            # shutDown
            ok( Cleanup(), 'cleanup' );
            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub Test_ReserveAmount {
    my $subTestName = (caller(0))[3];

    my $Order = LoadObjectByPath($ORDERPATH);
    my $hBaseURLParameters = {
        'User' => $Order->get('User'),
        'ScriptURL' => $Order->getSite->get('WebServerScriptUrl'),
        'ObjectID' => $Order->id(),
        'ActionType' => 'ChangeAction',
        'ActionName' => 'SendDataToKlarna',
    };
    my $hRequestParameters = {
        'Kl_Street' => 'Hellersbergstra\x{df}e', ## no critic (RequireInterpolationOfMetachars)
        'Kl_IsFromGermany' => 1,
        'SendDataToKlarna' => 'Pay',
        'Kl_HouseNo' =>14,
        'Kr_SSN' => 'skipped',
        'Kl_Birthdate' => '07/07/1960',
        'Kl_Gender' => 1,
        'Kr_CellPhone'=> '01522113356',
    };

    my $ReserveAmountResponse = _MakeRequest($hBaseURLParameters,$hRequestParameters);
    ok($ReserveAmountResponse->is_success,"$subTestName - called klarna 'ReserveAmount' webservice");

    my $ReservationNumber = $Order->get('LineItemContainer')->get('Payment')->get('ReservationNumber');
    like($ReservationNumber, qr/^[-+]?\d+$/, "$subTestName - Klarna ReservationNumber is correctly stored in LineItemPayment");
}

#----------------------------------------------------------------------------------------
sub Test_CancelReservation {
    my $subTestName = (caller(0))[3];

    my $Order = LoadObjectByPath($ORDERPATH);
    my $hBaseURLParameters = {
        'User' => LoadObjectByPath($SHOPPATH)->get('SuperMerchant'),
        'ScriptURL' => $Order->getSite->get('WebServerScriptAdminUrl'),
        'ObjectID' => $Order->id(),
        'ActionType' => 'ChangeAction',
        'ActionName' => 'MBO-CancelKlarnaInvoice',
    };
    my $hRequestParameters = {};

    my $CancelReservationResponse = _MakeRequest($hBaseURLParameters,$hRequestParameters);

    ok($CancelReservationResponse->is_success,"$subTestName - called klarna 'CancelReservation' webservice");
    is($Order->get('PaidOn'), undef, "$subTestName - PaidOn is set to undef");
    like($Order->get('CancelledOn'), qr#\d{2}(\d{2})?[-/.]\d{2}[-/.]\d{2}(\d{2})?[\$T ]#,"$subTestName - CancelledOn is set and is a Date");
}

#----------------------------------------------------------------------------------------
sub _SerialiseRequestParameters {
    my ($hRequestParameters) = @_;
    my $SerialisedRequestParameters = '';

    if (isValid($hRequestParameters)) {
        my @Params = ();
        foreach my $Keyname (keys %{$hRequestParameters}){
            push (@Params,join ('=', $Keyname,$hRequestParameters->{$Keyname}));
        }
        $SerialisedRequestParameters = join('&',@Params);
    }
    return $SerialisedRequestParameters;
}

#----------------------------------------------------------------------------------------
sub _SerialiseURL {
    my ($hParams) = @_;
    my $RequestURL = '';
    my $SessionID = InsertSession($hParams->{'User'}, 1);
    my $TokenNumber = GetInfoSessionBySessionID($SessionID)->{'Token'};
    my $JointActionParameter = $hParams->{'ActionType'}.'='.$hParams->{'ActionName'};
    my $URLParams = join('&'
        , 'ObjectID='.$hParams->{'ObjectID'}
        , $JointActionParameter
        , 'SessionID='.$SessionID
    );
    my $SecurityToken = 'sec'.$TokenNumber;
    $RequestURL = $hParams->{'ScriptURL'}.'/'.$SecurityToken.'/?'.$URLParams;
    return $RequestURL;
}

#----------------------------------------------------------------------------------------
sub _MakeRequest {
    my ($hBaseURLParameters,$hRequestParameters) = @_;

    my $FakeRequest = DE_EPAGES::WebInterface::API::FakeRequest->new;
    # overwrite the APPLICATION_SERVER constant with FakeRequestApplicationServer
    local $DE_EPAGES::WebInterface::API::ApplicationServer::APPLICATION_SERVER = $FakeRequest->{'AppServer'};
    my $RequestURL =  _SerialiseURL($hBaseURLParameters);
    my $SerialisedRequestParameters = _SerialiseRequestParameters($hRequestParameters);
    my $ContentType = 'application/x-www-form-urlencoded';
    my $Response = $FakeRequest->post( $RequestURL, $SerialisedRequestParameters, $ContentType );

    return $Response;
}

#----------------------------------------------------------------------------------------
sub Cleanup {
    return 1 if $ENV{'SKIP_CLEANUP'};
    LoadObjectByPath($ROOTPATH)->delete if ExistsObjectByPath($ROOTPATH);
    return 1;
}

#----------------------------------------------------------------------------------------
sub Init {
    return 1 if ExistsObjectByPath($ROOTPATH) && $ENV{'SKIP_CLEANUP'};
    Cleanup();
    LoadRootObject()->insertChild( 'Object', { Alias => 'Test' } );
    ImportObjects( 'KlarnaMockShop.xml', $ROOTPATH );
    return 1;
}
RunScript( Sub => \&Main );

1;