#========================================================================================
# package      DE_EPAGES::Klarna::t::CheckOrderStatus_internal
# state        public
#----------------------------------------------------------------------------------------
# description  internal unit tests of Klarna API CheckOrderStatus
#========================================================================================
package CheckOrderStatus_internal;
use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError );
use DE_EPAGES::Core::API::File qw ( GetFileContent );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Klarna::API::KlarnaOperations::CheckOrderStatus;
use DE_EPAGES::Klarna::UI::BasketOrder;
use DE_EPAGES::Test::Mock::EpagesObject;
use LWP::UserAgent;
use Test::MockObject::Extends;
use Test::More;

use constant INVOICE_NUMBER => 1000571580434;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            testRequestReservation();
            testRequestInvoice();

            testPositiveResponse();
            testFaultResponse();
            testHTTPErrorResponse();

            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub testRequestReservation {
    my $subTestName = (caller(0))[3];
    my $IsInvoiceNo = 0;

    my $LineItemPayment = _createLineItemPayment($IsInvoiceNo);
    my $PointerToLWPUserAgentObject = _createLWPUserAgent('checkOrderStatus_positiv_response.xml');
    my $CheckOrderStatus = _createCheckOrderStatus($LineItemPayment,$PointerToLWPUserAgentObject);
    $CheckOrderStatus->call();

    my $RequestedContent = $PointerToLWPUserAgentObject->{'HTTPRequest'}->content;
    my $ExpectedContent = _getExpectedContentReservation();
    is($RequestedContent,$ExpectedContent
        ,"$subTestName - compare the request structures whith reservation number"
    );
}

#----------------------------------------------------------------------------------------
sub testRequestInvoice {
    my $subTestName = (caller(0))[3];
    my $IsInvoiceNo = 1;

    my $LineItemPayment = _createLineItemPayment($IsInvoiceNo);
    my $PointerToLWPUserAgentObject = _createLWPUserAgent('checkOrderStatus_positiv_response.xml');
    my $CheckOrderStatus = _createCheckOrderStatus($LineItemPayment,$PointerToLWPUserAgentObject);
    $CheckOrderStatus->call();

    my $RequestedContent = $PointerToLWPUserAgentObject->{'HTTPRequest'}->content;
    my $ExpectedContent = _getExpectedContentInvoice();
    is($RequestedContent,$ExpectedContent
        ,"$subTestName - compare the request structures with invoice number"
    );
}

#----------------------------------------------------------------------------------------
sub _createCheckOrderStatus {
    my ($LineItemPayment, $LWPUserAgent) = @_;
    my $CheckOrderStatusObject = DE_EPAGES::Klarna::API::KlarnaOperations::CheckOrderStatus->new($LineItemPayment, $LWPUserAgent);
    my $CheckOrderStatusObjectMock = Test::MockObject::Extends->new( $CheckOrderStatusObject );
    $CheckOrderStatusObjectMock->mock('_LoadRootObject', sub{return _createSystemObject();}); #overwrite LoadRootObject
    return $CheckOrderStatusObjectMock;
}

#----------------------------------------------------------------------------------------
sub testPositiveResponse {
    my $subTestName = (caller(0))[3];

    my $LineItemPayment = _createLineItemPayment();
    my $PointerToLWPUserAgentObject = _createLWPUserAgent('checkOrderStatus_positiv_response.xml');

    my $CheckOrderStatus = _createCheckOrderStatus($LineItemPayment, $PointerToLWPUserAgentObject);
    my $hResponse = $CheckOrderStatus->call();

    my $ExpectedResponse = _getExpectedPositiveResponse();
    is_deeply($hResponse, $ExpectedResponse, "$subTestName - positive response (checkOrderStatus_positiv_response.xml)");
}

#----------------------------------------------------------------------------------------
sub testFaultResponse {
    my $subTestName = (caller(0))[3];

    my $LineItemPayment = _createLineItemPayment();
    my $PointerToLWPUserAgentObject = _createLWPUserAgent('fault_ unknown_order _response.xml');

    my $CheckOrderStatus = _createCheckOrderStatus($LineItemPayment, $PointerToLWPUserAgentObject);
    my $hResponse = $CheckOrderStatus->call();

    my $ExpectedResponse = _getExpectedFaultResponse();

    is_deeply($hResponse, $ExpectedResponse, "$subTestName - fault response (fault_ unknown_order _response.xml)");
}

#----------------------------------------------------------------------------------------
sub testHTTPErrorResponse {
    my $subTestName = (caller(0))[3];

    my $LineItemPayment = _createLineItemPayment();
    my $PointerToLWPUserAgentObject = _createLWPUserAgent();

    my $CheckOrderStatus = _createCheckOrderStatus($LineItemPayment, $PointerToLWPUserAgentObject);
    eval {
        $CheckOrderStatus->call();
    };
    if (ExistsError()) {
        my $Error = GetError();
        like ($Error->shortMessage(), qr/File does not exist:/, "$subTestName - HTTP error response");
    }
}

#========================================================================================
# DAL Wrapper
#========================================================================================
sub _createSystemObject{
    my $hSystemAttributes = {
    'EpagesVersion' => '6.17.0',
    'ProviderID' => 'epages',
    'StoreName' => 'Store',
    'KlarnaProtocolVersion' => '4.1',
    'KlarnaHostTest' => 'http://payment.testdrive.klarna.com',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::Site',$hSystemAttributes);
}

#----------------------------------------------------------------------------------------
sub _createShopObject {
        my $Attributes = {
        'IsInternalTestShop' => 1,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Store::API::Object::Store',$Attributes);
}

#----------------------------------------------------------------------------------------
sub _createLineItemPayment {
    my ( $IsInvoiceNo ) = @_;

    my $Attributes = {
        'ReservationNumber' => 2347271110,
        'PaymentMethod' => _createPaymentMethod(),
        'Kr_InvoiceNo' => undef,
    };

    if ($IsInvoiceNo){
        $Attributes->{'Kr_InvoiceNo'} = INVOICE_NUMBER;
    }

    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer',$Attributes);
}

#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
        my $Attributes = {
            'IsInTestMode' => 1,
            'Kr_StoreID' => 301,
            'Kr_SecretKey' => 'NoKn3kdeMKhxVmP',
            'PaymentType' => _createPaymentType(),
            'Site' => _createShopObject(),
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaInvoice',$Attributes);
}

#----------------------------------------------------------------------------------------
sub _createPaymentType {
        my $Attributes = {
        'Alias' => 'KlarnaInvoice'
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Payment::API::Attributes::System',$Attributes);
}

#========================================================================================
# Webservice Wrapper
#========================================================================================
sub _createLWPUserAgent {
    my ($RequestFile) = @_;
    my $GetTestResponsePath = GetKlarnaTestResponseDirectory();
    my $LWPUserAgent = LWP::UserAgent->new();
    my $MockedUserAgent = Test::MockObject::Extends->new( $LWPUserAgent );
    $MockedUserAgent->mock('request',sub{
                my $self = shift;
                my ($HTTPRequest) = @_;
                $MockedUserAgent->{'HTTPRequest'} = $HTTPRequest;
                return $self;
        });
    $MockedUserAgent->mock('content',sub{
            my $self = shift;
            my ($Params) = @_;
            if (isValid($RequestFile)){
                return ${GetFileContent($GetTestResponsePath.$RequestFile)};
            }else{
                return " \n";
            }
        });
    return $MockedUserAgent;
}

#----------------------------------------------------------------------------------------
sub GetKlarnaTestResponseDirectory {
    my $CartridgePackage = 'DE_EPAGES::Klarna';
    $CartridgePackage =~ s/::/\//g;
    return "$ENV{EPAGES_CARTRIDGES}/$CartridgePackage/t/TestResponses/";
}

#----------------------------------------------------------------------------------------
sub _getExpectedContentReservation {
    my $ExpectedContent = <<"REQUEST";
<?xml version="1.0" encoding="iso-8859-1"?>
<methodCall>
  <methodName>check_order_status</methodName>
  <params>
    <param>
      <value>
        <string>4.1</string>
      </value>
    </param>
    <param>
      <value>
        <string>perl:api:2.3.0:epages:6.17.0:epages,Store</string>
      </value>
    </param>
    <param>
      <value>
        <int>301</int>
      </value>
    </param>
    <param>
      <value>
        <string>L+Lk7hsqUO1CsdWksVrmyg==</string>
      </value>
    </param>
    <param>
      <value>
        <string>2347271110</string>
      </value>
    </param>
    <param>
      <value>
        <int>0</int>
      </value>
    </param>
  </params>
</methodCall>
REQUEST
;
return $ExpectedContent;
}

#----------------------------------------------------------------------------------------
sub _getExpectedContentInvoice {
    my $ExpectedContent = <<"REQUEST";
<?xml version="1.0" encoding="iso-8859-1"?>
<methodCall>
  <methodName>check_order_status</methodName>
  <params>
    <param>
      <value>
        <string>4.1</string>
      </value>
    </param>
    <param>
      <value>
        <string>perl:api:2.3.0:epages:6.17.0:epages,Store</string>
      </value>
    </param>
    <param>
      <value>
        <int>301</int>
      </value>
    </param>
    <param>
      <value>
        <string>VDiKaciR1c3rIbex7oClFg==</string>
      </value>
    </param>
    <param>
      <value>
        <string>1000571580434</string>
      </value>
    </param>
    <param>
      <value>
        <int>0</int>
      </value>
    </param>
  </params>
</methodCall>
REQUEST
;
return $ExpectedContent;
}

#----------------------------------------------------------------------------------------
sub _getExpectedPositiveResponse {
    my $ExpectedPositiveResponse = {
          'StatusCode' => 'OK',
          'Message' =>  {
                            'int' => '1'
                        }
    };
return $ExpectedPositiveResponse;
}

#----------------------------------------------------------------------------------------
sub _getExpectedFaultResponse {
    my $ExtectedFaultResponse = {
          'StatusCode' => '9131',
          'Message' => 'A problem has occured in the communication to Klarna that handles our invoicing. ' .
            'Please contact Klarna (http://www.klarna.com) for more information or choose another payment method.'
        };
return $ExtectedFaultResponse;
}

RunScript( Sub => \&Main );
1;