#========================================================================================
# package      DE_EPAGES::Klarna::t::Update_internal
# state        public
#----------------------------------------------------------------------------------------
# description  Internal unit tests for Klarna API Update
#========================================================================================
package Update_internal;
use strict;
use DE_EPAGES::Core::API::Error qw ( ExistsError GetError Error );
use DE_EPAGES::Core::API::PerlTools qw ( isValid );
use DE_EPAGES::Core::API::Script qw ( RunScript );
use DE_EPAGES::Database::API::Connection qw ( GetTestSite RunOnStore );
use DE_EPAGES::Klarna::API::KlarnaOperations::UpdateReservation;
use DE_EPAGES::Klarna::Hooks::Order qw ( OnBeforeEditOrder2Order );
use DE_EPAGES::Klarna::UI::BasketOrder;
use DE_EPAGES::Test::Mock::EpagesObject;
use LWP::UserAgent;
use Test::MockObject::Extends;
use Test::MockObject;
use Test::More;
use XMLRPC::Lite;

sub Main {
    RunOnStore(
        'Store' => GetTestSite(),
        'Sub'   => sub {

            testOrderHook();
            testPositiveResponse();
            testFaultResponse();
            testHTTPErrorResponse();

            done_testing();
        }
    );
}

#----------------------------------------------------------------------------------------
sub testOrderHook {
    my $subTestName = (caller(0))[3];

    my $Order = _createOrder();
    my $hParams = {
        'Order' => $Order,
    };

    OnBeforeEditOrder2Order($hParams);
    my $LineItemPaymentKlarna = $Order->get('LineItemContainer')->get('Payment');
    my $IsReservationOutOfDate = $LineItemPaymentKlarna->get('IsReservationOutOfDate');
    is($IsReservationOutOfDate, 1, "$subTestName - check Hook functionality")

}

#----------------------------------------------------------------------------------------
sub testPositiveResponse {
    my $subTestName = (caller(0))[3];

    my $LineItemContainer = _createLineItemContainer();
    my $Update = _createUpdate($LineItemContainer);
    my $RequestedContent = $Update->call();
    my $ExpectedContent = _getExpectedPositiveContent();
    is_deeply($RequestedContent, $ExpectedContent, "$subTestName - compare the request structure positive response");

}

#----------------------------------------------------------------------------------------
sub testFaultResponse {
    my $subTestName = (caller(0))[3];

    my $LineItemContainer = _createLineItemContainer();
    my $ResponseType = 'FaultResponse';
    my $Update = _createUpdate($LineItemContainer, $ResponseType);
    my $RequestedContent = $Update->call();
    my $ExpectedContent = _getExpectedNegativeContent();
    is_deeply($RequestedContent, $ExpectedContent, "$subTestName - compare the request structureb fault response");
}

#----------------------------------------------------------------------------------------
sub testHTTPErrorResponse {
    my $subTestName = (caller(0))[3];

    my $LineItemContainer = _createLineItemContainer();
    my $ResponseType = 'HttpError';
    my $Update = _createUpdate($LineItemContainer, $ResponseType);

    eval {
        $Update->call();
    };
    if (ExistsError()) {
        my $Error = GetError();
        like ($Error->code(), qr/HTTP Error/, "$subTestName - HTTP error response");
    }
}

#----------------------------------------------------------------------------------------
sub _createUpdate {
    my ($LineItemContainer, $ResponseType) = @_;

    my $PointerToXmlRpcClient = _createXmlRpcClient($ResponseType);
    my $UpdateObject = DE_EPAGES::Klarna::API::KlarnaOperations::UpdateReservation->new( $LineItemContainer, $PointerToXmlRpcClient );

    return $UpdateObject;
}

#========================================================================================
# DAL Wrapper
#========================================================================================
#----------------------------------------------------------------------------------------
sub _createOrder {

    my $Attributes = {
        'Alias' => '9999',
    };

    my $Order = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $Attributes);
    my $LineItemContainer = _createLineItemContainer($Order);
    $Order->set({
        'LineItemContainer' => $LineItemContainer,
    });

    return $Order;
}

#----------------------------------------------------------------------------------------
sub _createLineItemContainer {
    my ( $Order ) = @_;

    if (!isValid($Order)) {
        my $OrderAttributes = {
            'Alias' => '9999',
        };
        $Order = DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::CustomerOrder', $OrderAttributes);
    }

    my $LineItemPayment = _createLineItemPaymentKlarna();
    my $LineItemProductWithPiece = _createLineItemProduct('piece');
    my $LineItemProductWithoutPiece = _createLineItemProduct('pair');
    my $LineItemShipping = _createLineItemShipping();

    my $Attributes = {
        'LanguageID'    => '1',
        'LineItems'     => [$LineItemProductWithPiece, $LineItemProductWithoutPiece],
        'Parent'        => $Order,
        'Payment'       => $LineItemPayment,
        'Shipping'      => $LineItemShipping,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemContainer', $Attributes);
}

#----------------------------------------------------------------------------------------
sub _createLineItemPaymentKlarna {
    my $Attributes = {
        'Alias' => 'LineItemPaymentKlarna_52E66BB3-9A18-19B0-4752-AC1780026825',
        'BasePrice' => undef,
        'Discount' => undef,
        'instanceOf' => 'LineItemPaymentKlarna',
        'IsTestTransaction' => 1,
        'IsReservationOutOfDate' => 0,
        'LineItemPrice' => 1.95,
        'NameOrAlias' => 'Klarna invoice',
        'PaymentMethod' => _createPaymentMethod(),
        'Quantity' => 1,
        'ReservationNumber' => 2347010070,
        'SKU' => undef,
        'TaxRate' => 0.19,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Klarna::API::Object::LineItemPaymentKlarna', $Attributes);
}

#----------------------------------------------------------------------------------------
sub _createLineItemProduct {
    my ( $OrderUnitType ) = @_;

    my $Product = _createProduct($OrderUnitType);
    my $hLineItemProduct = {
        'Alias' => 'LineItemProduct_52F2241A-910A-3590-DB9E-AC178001885E ',
        'BasePrice' => 199.95,
        'Discount' => undef,
        'instanceOf' => 'LineItemProduct',
        'LineItemPrice' => 399.9,
        'NameOrAlias' => 'Campingaz Twister 270 - Milestones',
        'Product' => $Product,
        'Quantity' => 2,
        'SKU' => 'be_4040104003',
        'TaxRate' => 0.19,
    };

    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Klarna::API::Object::LineItemPaymentKlarna', $hLineItemProduct);
}

#----------------------------------------------------------------------------------------
sub _createLineItemShipping {
    my $Attributes = {
        'Alias' => 'LineItemShipping_52E66BB2-32C4-426E-C029-AC17800268F',
        'BasePrice' => undef,
        'Discount' => undef,
        'instanceOf' => '',
        'LineItemPrice' => 7,
        'NameOrAlias' => 'Standard delivery',
        'Quantity' => 1,
        'SKU' => undef,
        'TaxRate' => 0.19,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Order::API::Object::LineItemShipping', $Attributes);
}

#----------------------------------------------------------------------------------------
sub _createProduct {
    my ( $OrderUnitType ) = @_;

    my $Attributes = {
        'Alias' => 'be_4040104003',
        'BasePrice' => 199.95,
        'DisplayTitle' => 'Campingaz Twister 270 - Milestones',
        'LineItemPrice' => 399.9,
        'NameOrAlias' => 'Campingaz Twister 270 - Milestones',
        'OrderUnit' => _createUnitOfMesurement($OrderUnitType),
        'SKU' => 'be_4040104003',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Product::API::Object::Product', $Attributes);
}

#----------------------------------------------------------------------------------------
sub _createUnitOfMesurement {
    my ( $OrderUnitType ) = @_;

    my $Alias = $OrderUnitType;

    my $Attributes = {
        'Alias' => $Alias,
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Object::API::Object::UnitOfMeasurement', $Attributes);
}

#----------------------------------------------------------------------------------------
sub _createPaymentMethod {
    my $Attributes = {
        'IsInTestMode' => 1,
        'Kr_StoreID' => 301,
        'Kr_SecretKey' => 'NoKn3kdeMKhxVmP',
    };
    return DE_EPAGES::Test::Mock::EpagesObject->new('DE_EPAGES::Klarna::API::Object::PaymentMethodKlarnaInvoice', $Attributes);
}

#========================================================================================
# Webservice Wrapper
#========================================================================================

sub _createXmlRpcClient {
    my ($ExpectedResponse) = @_;
    my $Client = XMLRPC::Lite->proxy('http://payment.testdrive.klarna.com');
    my $MockedClient = Test::MockObject::Extends->new( $Client );
    $MockedClient->mock('call',sub{

        my $callResponse = _createXmlRpcResponse( $ExpectedResponse );
        return $callResponse;
    });

    return $MockedClient;
}

sub _createXmlRpcResponse {
    my ($ExpectedResponse) = @_;
    my $XmlRpcResponseClient = Test::MockObject->new( );
    $XmlRpcResponseClient->mock('faultcode' ,sub {
        return '9114';
    });

    $XmlRpcResponseClient->mock('faultstring', sub {
        return 'An integration problem has occured between the Store and Klarna. Please contact the Store for more information or choose another payment method.';
    });

    $XmlRpcResponseClient->mock('result', sub {
        if ($ExpectedResponse eq 'FaultResponse'){
            return '';
        } elsif($ExpectedResponse eq 'HttpError'){
            Error('HTTP Error');
        } else {
            return 'ok';
        }
    });

    return $XmlRpcResponseClient;
}

#----------------------------------------------------------------------------------------
sub GetKlarnaTestResponseDirectory {
    my $CartridgePackage = 'DE_EPAGES::Klarna';
    $CartridgePackage =~ s/::/\//g;
    return "$ENV{EPAGES_CARTRIDGES}/$CartridgePackage/t/TestResponses/";
}

#----------------------------------------------------------------------------------------
sub _getExpectedPositiveContent {
    my $ExpectedContent = {
        'StatusCode' => 'OK',
    };
return $ExpectedContent;
}

#----------------------------------------------------------------------------------------
sub _getExpectedNegativeContent {
    my $ExpectedNegativeResponse = {
          'StatusCode' => '9114',
          'Message' => 'An integration problem has occured between the Store and Klarna. Please contact the Store for more information or choose another payment method.'
    };
return $ExpectedNegativeResponse;
}

RunScript( Sub => \&Main );
1;